import { BangleEditorState, SpecRegistry, corePlugins, } from '@bangle.dev/core';
import { Plugin, PluginKey } from 'prosemirror-state';
import { useContext, useEffect, useState } from 'react';
import { EditorViewContext } from './ReactEditor';
import { rafSchedule } from '@bangle.dev/js-utils';
const LOG = false;
let log = LOG ? console.log.bind(console, 'react/usePluginState') : () => { };
export function useEditorState(props) {
    if (props.plugins && typeof props.plugins !== 'function') {
        throw new Error('plugins error: plugins must be a function');
    }
    const [state] = useState(() => new BangleEditorState(props));
    return state;
}
export function useSpecRegistry(initialSpecs, initialSpecRegistry, options = {}) {
    const [specRegistry] = useState(() => {
        return initialSpecRegistry || new SpecRegistry(initialSpecs, options);
    });
    return specRegistry;
}
export function usePlugins(getPlugins = corePlugins) {
    if (typeof getPlugins !== 'function') {
        throw new Error('usePlugins error: getPlugins must be a function');
    }
    const [result] = useState(getPlugins);
    return result;
}
export function usePluginState(pluginKey, throttle = false) {
    const view = useEditorViewContext();
    const [state, setState] = useState(pluginKey.getState(view.state));
    useEffect(() => {
        log('Setup plugin', pluginKey);
        let _setState = setState;
        if (throttle) {
            _setState = rafSchedule(setState);
        }
        const plugin = watcherPlugin(pluginKey, _setState);
        view._updatePluginWatcher(plugin);
        return () => {
            if (throttle) {
                _setState.cancel();
            }
            view._updatePluginWatcher(plugin, true);
        };
    }, [view, pluginKey, throttle]);
    return state;
}
export function useEditorViewContext() {
    return useContext(EditorViewContext);
}
function watcherPlugin(pluginKey, setState) {
    return new Plugin({
        key: new PluginKey(`withPluginState_${pluginKey.key}`),
        view() {
            return {
                update(view, prevState) {
                    const { state } = view;
                    if (prevState === state) {
                        return;
                    }
                    const newPluginState = pluginKey.getState(state);
                    if (newPluginState !== pluginKey.getState(prevState)) {
                        setState(newPluginState);
                    }
                },
            };
        },
    });
}
